@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 # Redmine - project management software
-# Copyright (C) 2006-2019  Jean-Philippe Lang
+# Copyright (C) 2006-2021  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@ require File.expand_path('../../test_helper', __FILE__)
 class UsersControllerTest < Redmine::ControllerTest
   include Redmine::I18n
 
-  fixtures :users, :email_addresses, :projects, :members, :member_roles, :roles,
+  fixtures :users, :user_preferences, :email_addresses, :projects, :members, :member_roles, :roles,
            :custom_fields, :custom_values, :groups_users,
            :auth_sources,
            :enabled_modules,
@@ -68,19 +68,48 @@ class UsersControllerTest < Redmine::ControllerTest
 
   def test_index_csv
     with_settings :default_language => 'en' do
-      get :index, :params => { :format => 'csv' }
+      user = User.logged.status(1).first
+      user.update(passwd_changed_on: Time.current.last_month, twofa_scheme: 'totp')
+      get :index, params: {format: 'csv'}
       assert_response :success
 
       assert_equal User.logged.status(1).count, response.body.chomp.split("\n").size - 1
-      assert_include 'active', response.body
-      assert_not_include 'locked', response.body
+      assert_include format_time(user.updated_on), response.body.split("\n").second
+      assert_include format_time(user.passwd_changed_on), response.body.split("\n").second
+
+      # status
+      assert_include 'active', response.body.split("\n").second
+      assert_not_include 'locked', response.body.split("\n").second
+
+      # twofa_scheme
+      assert_include 'Authenticator app', response.body.split("\n").second
+      assert_include 'disabled', response.body.split("\n").third
+
+      assert_equal 'text/csv', @response.media_type
+    end
+  end
+
+  def test_index_csv_with_custom_field_columns
+    float_custom_field = UserCustomField.generate!(:name => 'float field', :field_format => 'float')
+    date_custom_field = UserCustomField.generate!(:name => 'date field', :field_format => 'date')
+    user = User.last
+    user.custom_field_values = {float_custom_field.id.to_s => 2.1, date_custom_field.id.to_s => '2020-01-10'}
+    user.save
+
+    User.find(@request.session[:user_id]).update(:language => nil)
+    with_settings :default_language => 'fr' do
+      get :index, :params => {:name => user.lastname, :format => 'csv'}
+      assert_response :success
+
+      assert_include 'float field;date field', response.body
+      assert_include '2,10;10/01/2020', response.body
       assert_equal 'text/csv', @response.media_type
     end
   end
 
   def test_index_csv_with_status_filter
     with_settings :default_language => 'en' do
-      get :index, :params => { :status => 3, :format => 'csv' }
+      get :index, :params => {:status => 3, :format => 'csv'}
       assert_response :success
 
       assert_equal User.logged.status(3).count, response.body.chomp.split("\n").size - 1
@@ -123,7 +152,7 @@ class UsersControllerTest < Redmine::ControllerTest
     get :show, :params => {:id => 2}
     assert_response :success
 
-    assert_select 'li[class=?]', 'cf_4', :text => /Phone number/
+    assert_select 'li.cf_4.string_cf', :text => /Phone number/
   end
 
   def test_show_should_not_display_hidden_custom_fields
@@ -233,6 +262,20 @@ class UsersControllerTest < Redmine::ControllerTest
     end
   end
 
+  def test_show_should_list_all_emails
+    EmailAddress.create!(user_id: 3, address: 'dlopper@example.net')
+    EmailAddress.create!(user_id: 3, address: 'dlopper@example.org')
+
+    @request.session[:user_id] = 1
+    get :show, params: {id: 3}
+
+    assert_select 'li', text: /Email:/ do
+      assert_select 'a:nth-of-type(1)', text: 'dlopper@somenet.foo'
+      assert_select 'a:nth-of-type(2)', text: 'dlopper@example.net'
+      assert_select 'a:nth-of-type(3)', text: 'dlopper@example.org'
+    end
+  end
+
   def test_new
     get :new
     assert_response :success
@@ -240,22 +283,25 @@ class UsersControllerTest < Redmine::ControllerTest
   end
 
   def test_create
-    Setting.bcc_recipients = '1'
-
-    assert_difference 'User.count' do
-      assert_difference 'ActionMailer::Base.deliveries.size' do
-        post :create, :params => {
-          :user => {
-            :firstname => 'John',
-            :lastname => 'Doe',
-            :login => 'jdoe',
-            :password => 'secret123',
-            :password_confirmation => 'secret123',
-            :mail => 'jdoe@gmail.com',
-            :mail_notification => 'none'
-          },
-          :send_information => '1'
-        }
+    with_settings :bcc_recipients => '1' do
+      assert_difference 'User.count' do
+        assert_difference 'ActionMailer::Base.deliveries.size' do
+          post(
+            :create,
+            :params => {
+              :user => {
+                :firstname => 'John',
+                :lastname => 'Doe',
+                :login => 'jdoe',
+                :password => 'secret123',
+                :password_confirmation => 'secret123',
+                :mail => 'jdoe@gmail.com',
+                :mail_notification => 'none'
+              },
+              :send_information => '1'
+            }
+          )
+        end
       end
     end
 
@@ -393,14 +439,25 @@ class UsersControllerTest < Redmine::ControllerTest
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
     assert_mail_body_match '0.0.0.0', mail
-    assert_mail_body_match I18n.t(:mail_body_security_notification_add, field: I18n.t(:field_admin), value: 'eschmoe'), mail
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_add,
+        field: I18n.t(:field_admin),
+        value: 'eschmoe'
+      ),
+      mail
+    )
     assert_select_email do
       assert_select 'a[href^=?]', 'http://localhost:3000/users', :text => 'Users'
     end
 
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.bcc, mail.cc].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -487,17 +544,21 @@ class UsersControllerTest < Redmine::ControllerTest
   end
 
   def test_update_with_activation_should_send_a_notification
-    u = User.new(:firstname => 'Foo', :lastname => 'Bar', :mail => 'foo.bar@somenet.foo', :language => 'fr')
+    u = User.new(:firstname => 'Foo', :lastname => 'Bar',
+                 :mail => 'foo.bar@somenet.foo', :language => 'fr')
     u.login = 'foo'
     u.status = User::STATUS_REGISTERED
     u.save!
     ActionMailer::Base.deliveries.clear
-    Setting.bcc_recipients = '1'
-
-    put :update, :params => {
-      :id => u.id,
-      :user => {:status => User::STATUS_ACTIVE}
-    }
+    with_settings :bcc_recipients => '1' do
+      put(
+        :update,
+        :params => {
+          :id => u.id,
+          :user => {:status => User::STATUS_ACTIVE}
+        }
+      )
+    end
     assert u.reload.active?
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
@@ -507,13 +568,19 @@ class UsersControllerTest < Redmine::ControllerTest
 
   def test_update_with_password_change_should_send_a_notification
     ActionMailer::Base.deliveries.clear
-    Setting.bcc_recipients = '1'
-
-    put :update, :params => {
-      :id => 2,
-      :user => {:password => 'newpass123', :password_confirmation => 'newpass123'},
-      :send_information => '1'
-    }
+    with_settings :bcc_recipients => '1' do
+      put(
+        :update,
+        :params => {
+          :id => 2,
+          :user => {
+            :password => 'newpass123',
+            :password_confirmation => 'newpass123'
+          },
+         :send_information => '1'
+        }
+      )
+    end
     u = User.find(2)
     assert u.check_password?('newpass123')
 
@@ -525,24 +592,28 @@ class UsersControllerTest < Redmine::ControllerTest
 
   def test_update_with_generate_password_should_email_the_password
     ActionMailer::Base.deliveries.clear
-    Setting.bcc_recipients = '1'
-
-    put :update, :params => {
-      :id => 2,
-      :user => {
-        :generate_password => '1',
-        :password => '',
-        :password_confirmation => ''
-      },
-      :send_information => '1'
-    }
-
+    with_settings :bcc_recipients => '1' do
+      put(
+        :update,
+        :params => {
+          :id => 2,
+          :user => {
+            :generate_password => '1',
+            :password => '',
+            :password_confirmation => ''
+          },
+          :send_information => '1'
+        }
+      )
+    end
     mail = ActionMailer::Base.deliveries.last
     assert_not_nil mail
+    u = User.find(2)
+    assert_equal [u.mail], mail.bcc
     m = mail_body(mail).match(/Password: ([a-zA-Z0-9]+)/)
     assert m
     password = m[1]
-    assert User.find(2).check_password?(password)
+    assert u.check_password?(password)
   end
 
   def test_update_without_generate_password_should_not_change_password
@@ -619,11 +690,21 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_add, field: I18n.t(:field_admin), value: User.find(2).login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_add,
+        field: I18n.t(:field_admin),
+        value: User.find(2).login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.bcc, mail.cc].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -639,11 +720,21 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_remove, field: I18n.t(:field_admin), value: user.login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_remove,
+        field: I18n.t(:field_admin),
+        value: user.login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.bcc, mail.cc].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -659,17 +750,27 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_remove, field: I18n.t(:field_admin), value: User.find(2).login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_remove,
+        field: I18n.t(:field_admin),
+        value: User.find(2).login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.bcc, mail.cc].flatten.include?(admin.mail)
+        end
+      )
     end
 
     # if user is already locked, destroying should not send a second mail
     # (for active admins see furtherbelow)
     ActionMailer::Base.deliveries.clear
-    delete :destroy, :params => {:id => 1}
+    delete :destroy, :params => {:id => 1, :confirm => User.find(1).login}
     assert_nil ActionMailer::Base.deliveries.last
 
   end
@@ -685,11 +786,21 @@ class UsersControllerTest < Redmine::ControllerTest
     }
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_add, field: I18n.t(:field_admin), value: user.login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_add,
+        field: I18n.t(:field_admin),
+        value: user.login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.bcc, mail.cc].flatten.include?(admin.mail)
+        end
+      )
     end
   end
 
@@ -708,19 +819,56 @@ class UsersControllerTest < Redmine::ControllerTest
     assert_response 404
   end
 
+  def test_update_with_blank_email_should_not_raise_exception
+    assert_no_difference 'User.count' do
+      with_settings :gravatar_enabled => '1' do
+        put :update, :params => {
+          :id => 2,
+          :user => {:mail => ''}
+        }
+      end
+    end
+    assert_response :success
+    assert_select_error /Email cannot be blank/
+  end
+
   def test_destroy
     assert_difference 'User.count', -1 do
-      delete :destroy, :params => {:id => 2}
+      delete :destroy, :params => {:id => 2, :confirm => User.find(2).login}
     end
     assert_redirected_to '/users'
     assert_nil User.find_by_id(2)
   end
 
+  def test_destroy_with_lock_param_should_lock_instead
+    assert_no_difference 'User.count' do
+      delete :destroy, :params => {:id => 2, :lock => 'lock'}
+    end
+    assert_redirected_to '/users'
+    assert User.find_by_id(2).locked?
+  end
+
+  def test_destroy_should_require_confirmation
+    assert_no_difference 'User.count' do
+      delete :destroy, :params => {:id => 2}
+    end
+    assert_response :success
+    assert_select '.warning', :text => /Are you sure you want to delete this user/
+  end
+
+  def test_destroy_should_require_correct_confirmation
+    assert_no_difference 'User.count' do
+      delete :destroy, :params => {:id => 2, :confirm => 'wrong'}
+    end
+    assert_response :success
+    assert_select '.warning', :text => /Are you sure you want to delete this user/
+  end
+
   def test_destroy_should_be_denied_for_non_admin_users
     @request.session[:user_id] = 3
 
     assert_no_difference 'User.count' do
-      get :destroy, :params => {:id => 2}
+      delete :destroy, :params => {:id => 2, :confirm => User.find(2).login}
     end
     assert_response 403
   end
@@ -728,14 +876,16 @@ class UsersControllerTest < Redmine::ControllerTest
   def test_destroy_should_be_denied_for_anonymous
     assert User.find(6).anonymous?
     assert_no_difference 'User.count' do
-      put :destroy, :params => {:id => 6}
+      delete :destroy, :params => {:id => 6, :confirm => User.find(6).login}
     end
     assert_response 404
   end
 
   def test_destroy_should_redirect_to_back_url_param
     assert_difference 'User.count', -1 do
-      delete :destroy, :params => {:id => 2, :back_url => '/users?name=foo'}
+      delete :destroy, :params => {:id => 2,
+                                   :confirm => User.find(2).login,
+                                   :back_url => '/users?name=foo'}
     end
     assert_redirected_to '/users?name=foo'
   end
@@ -745,14 +895,47 @@ class UsersControllerTest < Redmine::ControllerTest
     user.admin = true
     user.save!
     ActionMailer::Base.deliveries.clear
-    delete :destroy, :params => {:id => user.id}
+    delete :destroy, :params => {:id => user.id, :confirm => user.login}
 
     assert_not_nil (mail = ActionMailer::Base.deliveries.last)
-    assert_mail_body_match I18n.t(:mail_body_security_notification_remove, field: I18n.t(:field_admin), value: user.login), mail
-
+    assert_mail_body_match(
+      I18n.t(
+        :mail_body_security_notification_remove,
+        field: I18n.t(:field_admin),
+        value: user.login
+      ),
+      mail
+    )
     # All admins should receive this
     User.where(admin: true, status: Principal::STATUS_ACTIVE).each do |admin|
-      assert_not_nil ActionMailer::Base.deliveries.detect{|mail| [mail.bcc, mail.cc].flatten.include?(admin.mail) }
+      assert_not_nil(
+        ActionMailer::Base.deliveries.detect do |mail|
+          [mail.bcc, mail.cc].flatten.include?(admin.mail)
+        end
+      )
+    end
+  end
+
+  def test_destroy_without_unsubscribe_is_denied
+    user = User.find(2)
+    user.update(admin: true) # Create other admin so self can be deleted
+    @request.session[:user_id] = user.id
+    with_settings unsubscribe: 0 do
+      assert_no_difference 'User.count' do
+        delete :destroy, params: {id: user.id}
+      end
+      assert_response 422
+    end
+  end
+
+  def test_destroy_last_admin_is_denied
+    user = User.find(1)
+    @request.session[:user_id] = user.id
+    with_settings unsubscribe: 1 do
+      assert_no_difference 'User.count' do
+        delete :destroy, params: {id: user.id}
+      end
+      assert_response 422
     end
   end
 end
